%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
%
% Part of softare library HSpec ("Hilbert Spectrum")
% Free software written by  Sofia Olhede and Andrew Walden,
% Dept. of Mathematics, Imperial College, London SW7 2BZ, UK
% November 2004
%
% For terms of use and warranty disclaimer please refer to the
% README file
%
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%
% 
% this creates Fig 1(c) or 1(d) of 
% "A generalized demodulation approach to time-frequency projections for
% multicomponent signals" by S. Olhede and A.T. Walden
%  Proc R. Soc. Lond. A., under revision.
%
% it carries out steps 1-4 and 6 of the algorithm, 
% i.e., it does forward demodulation only (step 5 not included)
% if demod is set to 0 below,
% and carries out steps 1-6 (full algorithm) if demod set to 1
%
%
clf; clear all;
%demod=0; % if only forward demodulation (Fig 1c)
demod=1; % if complete algorithm (Fig 1d)
batsignal;
signal=transpose(s);
J0=3;
deltat=7e-6;
t=(0:399).*deltat;
kfilt=0;% FK(18)
%
b2=-0.0003;
b1=0.175;
shift=b2.*(t./deltat).^2+b1.*(t./deltat);
dsh=(2.*b2.*(t./deltat)+b1)./deltat;

dvector=exp(-i.*2.*pi.*shift);
%
% step 1 of algorithm
%
y=hilbert(signal); % make signal analytic
%
% step 2 of algorithm
%
y=y.*dvector; % apply demodulation vector
%
% step 3 of algorithm
%
z=canalytic(y); % render new complex signal analytic
if demod==0
% for forward demodulation only set dvector to ones
	dvector=ones; 
	demodhspecmodwpt(z,J0,kfilt,deltat,t,dvector);
else 
% forward and reverse demodulation
	demodhspecmodwpt(z,J0,kfilt,deltat,t,dvector);
end
