%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
%
% Part of softare library HSpec ("Hilbert Spectrum")
% Free software written by  Sofia Olhede and Andrew Walden,
% Dept. of Mathematics, Imperial College, London SW7 2BZ, UK
% November 2004
%
% For terms of use and warranty disclaimer please refer to the
% README file
%
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% 
% this creates Fig 2(b) of 
% "A generalized demodulation approach to time-frequency projections for
% multicomponent signals" by S. Olhede and A.T. Walden
%
%  Proc R. Soc. Lond. A.
%
% Author: AT Walden
%
clf; clear all;
t=0:1023;
J0=3;
kfilt=0;% FK(18)
deltat=1;
% create the artificial whale signal
inst1=(-1.2783./(2.*pi)).*10.^(-6).*(t-512).^3 + (1.3823./(2.*pi)).*t;
inst1(1:250)=zeros;
inst2=(-1.2566./(2.*pi)).*10^(-6).*t.^3 + (0.0019./(2.*pi)).*t.^2 +(1.1983./(2.*pi)).*t;
signal=sin(2.*pi.*inst1)+sin(2.*pi.*inst2);
%
% we now match s_l'=3b_3 l^2 + 2b_2 l +b1 at l=0, 500 and 1000
% to f=0.125, 0.28125; 0.14 to get parameters b_1, b_2, b_3

M=[3 2 1; 3.*(500.^2) 2.*500 1; 3.*(1000.^2) 2.*1000 1];
R=[0.125; 0.28125; 0.14];
b=inv(M)*R;
b3=b(1); b2=b(2); b1=b(3);
%
% the derivative of shift has the correct curvature but goes through the points
% given above; so we merely shift up or down by modifying b1 by .21
%
shift=b3.*t.^3+ b2.*t.^2+(b1-.21).*t;
%

figure(1)
hilbert_MODWPT(signal,J0,kfilt,deltat,t);

dsh=3.*b3.*t.^2+ 2.*b2.*t+b1-.21;
fb1=(1./(16.*deltat))+dsh;
fb2=(2./(16.*deltat))+dsh;
fb3=(3./(16.*deltat))+dsh;
fb4=(4./(16.*deltat))+dsh;
fb5=(5./(16.*deltat))+dsh;
fb6=(6./(16.*deltat))+dsh;
hold on
plot(t,fb1,'--')
hold on
plot(t,fb2,'--')
hold on
plot(t,fb3,'--')
hold on
plot(t,fb4,'--')
hold on
plot(t,fb5,'--')
hold on
plot(t,fb6,'--')



