%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
%
% Part of softare library HSpec ("Hilbert Spectrum")
% Free software written by  Sofia Olhede and Andrew Walden,
% Dept. of Mathematics, Imperial College, London SW7 2BZ, UK
% November 2004
%
% For terms of use and warranty disclaimer please refer to the
% README file
%
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% 
% this creates Fig 2(c) or 2(d) of 
% "A generalized demodulation approach to time-frequency projections for
% multicomponent signals" by S. Olhede and A.T. Walden
%
% it carries out steps 1-4 and 6 of the algorithm, 
% i.e., it does forward demodulation only (step 5 not included)
% if demod is set to 0 below,
% and carries out steps 1-6 (full algorithm) if demod set to 1
%
%  Proc R. Soc. Lond. A.
%
% Author: AT Walden
%
clf; clear all;
%demod=0; % if only forward demodulation (Fig 2c)
demod=1; % if complete algorithm (Fig 2d)
t=0:1023;
J0=3;
kfilt=0;% filter FK(18)
deltat=1;
% calculate artificial whale vocalization:
a1=-1.2566.*10^(-6);
a2=0.0019;
a3=1.1983;
a4=-1.2783.*10.^(-6);
a5=1.3823;
phase1=a4.*(t-512).^3 + a5.*t;
phase1(1:250)=zeros;
amp1(1:1024)=zeros;
nr=1024-250;% number non-zero
amp1(251:1024)=ones;
amp2(1:1024)=ones;
phase2=a1.*t.^3 + a2.*t.^2 +a3.*t;
signal=amp1.*sin(phase1)+amp2.*sin(phase2);

tl=0:1023;
% "inst freq" from phase (not analytic inst freq)
instfa=(3.*a1.*tl.^2+2.*a2.*tl+a3)./(2.*pi);
tll=250:1023;
instfb=(3.*a4.*(tll-512).^2+a5)./(2.*pi);

%
% we now match s_l'=3b_3 l^2 + 2b_2 l +b1 at l=0, 500 and 1000
% to f=0.125, 0.28125; 0.14 to get parameters b_1, b_2, b_3

M=[3 2 1; 3.*(500.^2) 2.*500 1; 3.*(1000.^2) 2.*1000 1];
R=[0.125; 0.28125; 0.14];
b=inv(M)*R;
b3=b(1); b2=b(2); b1=b(3);
%
% the derivative of shift has the correct curvature but goes through the points
% given above; so we merely shift up or down by modifying b1
%
shift=b3.*t.^3+ b2.*t.^2+(b1-.21).*t;
dsh=3.*b3.*t.^2+ 2.*b2.*t+b1-.21;

dvector=exp(-i.*2.*pi.*shift);
% step 1
y=hilbert(signal); % make signal analytic
% step 2
y=y.*dvector; % apply demodulation vector
% step 3
z=canalytic(y); % render new complex signal analytic
if demod==0
% for forward demodulation only set dvector to ones
	dvector=ones;
	demodhspecmodwpt(z,J0,kfilt,deltat,t,dvector)
	plot(0:1023, instfa,'-.');% inst freqs of known artificial signal
	plot(250:1023, instfb,'-.');
else % full algorithm
% forward and reverse demodulation
	demodhspecmodwpt(z,J0,kfilt,deltat,t,dvector)
	plot(0:1023, instfa,'-.');
	plot(250:1023, instfb,'-.');
end

