%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
%
% Part of softare library HSpec ("Hilbert Spectrum")
% Free software written by  Sofia Olhede and Andrew Walden,
% Dept. of Mathematics, Imperial College, London SW7 2BZ, UK
% November 2004
%
% For terms of use and warranty disclaimer please refer to the
% README file
%
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%
function val=covphi(At,As,phit,phis,t,s,comp,j,k,sigma)
%
% computes covariance of instantaneous phase as set out in
%
% "A generalized demodulation approach to time-frequency projections for
% multicomponent signals" by S. Olhede and A.T. Walden
%  Proc R. Soc. Lond. A., under revision.
%
% At: inst ampl at time t
% As: inst ampl at s
% phit: inst phase at t
% phis: inst phase at s
% comp: 1 for MODWPT details
%
v=length(sigma);
if v==1
sigma=sigma.*ones(1,length(At));
end;
%
% this function estimates the covariance 
if comp==1
% cov of coeffs of real/imag part of complex projection using MODWPT
   cov1=covardetailp(j,k,t-s);
% cross-cov between real and imag parts of same 
   cov1h=covardetailph(j,k,t-s);
end;
%
% if the complex projection arose from a single analytic transform
% on a real input, the only factor to apply here is the variance
% but if it arose after TWO analytic transforms such as happens with
% demodulation, a further factor of four is required (as shown in Olhede
% and Walden demodulation paper). We assume in this latter case that
% the factor of 4 will be applied outside this subroutine
%
val=sigma.^2.*(cov1.*cos(phit-phis)+cov1h.*sin(phit-phis))./(At.*As);
