%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
%
% Part of softare library HSpec ("Hilbert Spectrum")
% Free software written by  Sofia Olhede and Andrew Walden,
% Dept. of Mathematics, Imperial College, London SW7 2BZ, UK
% November 2004
%
% For terms of use and warranty disclaimer please refer to the
% README file
%
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%
function vektor=demodhspecmodwpt(x,J0,kfilt,deltat,t,dvector)
%
%  carries out steps 4, 5 and 6 of the algorithm
%
%  MODWPT details, reverse demodulation, and Hilbert spectrum
%
% note if dvector is ones, the reverse demodulation is not carried out
%
%	x		complex input formed as follows
%           take original signal and make analytic with hilbert2
%           apply e^{-ip(t)} via mult with dvector
%           render the complex result analytic using canalytic
%	J0		max scale (1 to J0)
%	kfilt	selects low-pass filter LA, FK etc as defined
%			in wavgfilts
%	deltat	sample interval
%	t		time vector matching x
%   dvector e^{-ip(t)}, we will apply its conjugate to details
%
%
% step 4: Construct MODWPT detail sequences
%
[W] = modwptdetgenr(x, kfilt, J0);
n=length(x);
%
% select level J0 packets 
%
% pull out the wavelet packet details using index
% (note the low frequency one is excluded)
%
indexy(1:2^(J0+1)-1)=zeros;

l=0;
for j=2^J0+1:2^(J0+1)-1
	l=l+1;
% step 5 of the algorithm:
%
% now apply e^{ip(t)}, i.e., reverse demodulation
%
	vector(l,:)=W(j,:).*conj(dvector);
	indexy(j)=1;
end
% 
ndets=size(vector,1) % number of detail sequences
% 

% step 6 of the algorithm:

for j=1:ndets
	vector2(j,:) = vector(j,:); % vector already  analytic signal
	vector3(j,:)=imag(vector(j,:)).^2+real(vector(j,:)).^2; % mod squared
%	vectorkonst(j,:)=sqrt(vector3(j,:)); % mod or amplitude
	phi(j,:)=angle(vector2(j,:));
	phi(j,:)=unwrap(phi(j,:));
	phi2(j,:)=numericaldiff(phi(j,:),deltat)./(2*pi);% inst freq
	vector4(j,:)=phi2(j,:).*vector3(j,:);
end;

%
% plot Hilbert spectrum
%
nf=512;
tf(nf,n)=zeros;
fnyq=1/(2*deltat);
fi=linspace(0,fnyq,nf);
lint=fnyq./(nf-1);% interval of freq grid
%
% map inst freq onto regular frequency grid
%
for k=1:n
	for j=1:ndets
		if( phi2(j,k) > 0 & phi2(j,k) <= fnyq)
			l=round(phi2(j,k)./lint)+1;
			tf(l,k)=tf(l,k)+vector3(j,k);% add all the amplitudes^2
		end
	end
end
s=max(max(tf));
vs2=linspace(0,s,10);

figure(1)
colormap(1-gray(256))
contour(t,fi,tf,vs2)
hold on
for j=1:size(W,1)
	sjj=floor(log2(j));
	if indexy(j)==1
		plot(t,ones(1,length(t)).*(j-2^(sjj))./2^(sjj+1)...
		./deltat,'k:',t,ones(1,length(t)).*(j+1-2^(sjj))...
		./2^(sjj+1)./deltat,'k:')
	end;
end;
hold on
axis([min(t) max(t) 0 fnyq])
ylabel('frequency')
xlabel('time')

% for nice labelling of bat data:
%
%xlabel('time (s)') % for bat data
%set(gca,'xtick', [0 0.001 0.002]) % for bat data
%set(gca,'xticklabel', '0| 0.001| 0.002')% for bat data

box on

