%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
%
% Part of softare library HSpec ("Hilbert Spectrum")
% Free software written by  Sofia Olhede and Andrew Walden,
% Dept. of Mathematics, Imperial College, London SW7 2BZ, UK
% November 2004
%
% For terms of use and warranty disclaimer please refer to the
% README file
%
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%
function matrix=modwptdetgenr(X,kfilt,J);
%
% constructs the detail sequences from MODWPT
%
% (i) Does MODWPT to level J, storing the tree in W
%     where W=modwptgen(X,kfilt,J,0)
% (ii)Cross-correlates each wavelet packet with appropriate filter
%
% details stored in matrix as follows
%  global node 
%  index
%
%   1                X
%   2                {\cal D}_{1,0,t}  (j=1,n=0)
%   3                {\cal D}_{1,1,t}  (j=1,n=1)
%   4                {\cal D}_{2,0,t}  (j=2,n=0)
%   5                {\cal D}_{2,1,t}  (j=2,n=1)
%                      ..
%   7                {\cal D}_{2,3,t}
%   8                {\cal D}_{3,0,t}
%                       ..
%   15                {\cal D}_{3,7,t}  etc

U=fliplr(modwptgenr(X,kfilt,J,0));
matrix(1,:)=X;
for k=2:2^(J+1)-1
	j=floor(log2(k));
	S=modwptgenr(U(k,:),kfilt,j,0);
	matrix(k,:)=fliplr(S(k,:));
	clear S;
end;
