%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
%
% Part of softare library HSpec ("Hilbert Spectrum")
% Free software written by  Sofia Olhede and Andrew Walden,
% Dept. of Mathematics, Imperial College, London SW7 2BZ, UK
% November 2004
%
% For terms of use and warranty disclaimer please refer to the
% README file
%
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%
function [phi,phi2,amp,vector]=phasefreqmodwpt(x,J0,kfilt,deltat,t)
%	x		complex input formed as follows
%           take original signal and make analytic with hilbert2
%	J0		max scale (1 to J0)
%	kfilt	selects low-pass filter LA, FK etc as defined
%			in wavgfilts
%	deltat	sample interval
%	t		time vector matching x
%
%   phi 	inst phase
%	phi2	inst freq
%	amp		amplitude of analytic vector
%	vector	the analytic vector
%   dvector e^{-ip(t)}, we will apply its conjugate to details
%
% computes MODWPT detail sequences and inst phase and frequency
% used in work to compute variance of inst freq estimate
%
% Construct MODWPT detail sequences
%
[W] = modwptdetgenr(x, kfilt, J0);
n=length(x);
%
% select level J0 packets 
%
% pull out the wavelet packet details using index 
% (note the low frequency one is excluded)
%
indexy(1:2^(J0+1)-1)=zeros;
l=0;
for j=2^J0+1:2^(J0+1)-1
	l=l+1;
	vector(l,:)=W(j,:);
	indexy(j)=1;
end
% 
ndets=size(vector,1) % number of detail sequences
% 

for j=1:ndets
	vector2(j,:) = vector(j,:); % vector already  analytic signal
	amp(j,:)=imag(vector(j,:)).^2+real(vector(j,:)).^2; % mod squared
	amp(j,:)=sqrt(amp(j,:));% mod (amplitude)
	phi(j,:)=angle(vector2(j,:));
	phi(j,:)=unwrap(phi(j,:));
	phi2(j,:)=numericaldiff(phi(j,:),deltat)./(2*pi);% inst freq
end;

