%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
%
% Part of softare library HSpec ("Hilbert Spectrum")
% Free software written by  Sofia Olhede and Andrew Walden,
% Dept. of Mathematics, Imperial College, London SW7 2BZ, UK
% November 2004
%
% For terms of use and warranty disclaimer please refer to the
% README file
%
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%
% this creates Fig. 3 of 
% "A generalized demodulation approach to time-frequency projections for
% multicomponent signals" by S. Olhede and A.T. Walden
%  Proc R. Soc. Lond. A., under revision.
%
% program defines a signal with quadratic phase and looks at the estimation of the
% phase derivative (`instantaneous frequency'). 
% We take J0=2 so that the frequency subbands
% are n=0, 1, 2, 3. the subroutine phasefreqmodwpt only looks at n=1..3.
% the signal resides in band 1 and we take band 3 (high freq) to be noise
% for estimation of noise variance
%
% does three things:
% (1) calcs variance of `inst freq' estimates using actual amp and phase
% (2) simulate variance of `inst freq' by repeated estimation
% (3) repeatedly estimate the same using estimated params in the formula
%
clear all;
clf;
%
N=1024;
ston=10;% signal-to-noise variance ratio=10
t=(0:1023);
f1=0.18;
f2=0.35;
f3=0.02;
%
signal=sin(f3.*pi.*t.^2./N+f1.*2.*pi.*t);% quadratic phase
%
s2=std(signal).^2;% `variance' of signal
sigmaepssq=s2./ston;% variance of noise
sigmaeps=sqrt(sigmaepssq)
%
randn('state',1);
%
J0=2;
deltat=1;
kfilt=0;% FK(18)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% step 1, var of inst freq using actual amp and phase
%
input=signal;% pure signal
dvector(1:length(t))=ones;% null demodulation
y=hilbert(input); % make  analytic
y=y.*dvector; % apply (null) demodulation vector
z=canalyticno2(y); % render new complex signal analytic
% %
[phi,phi2,amp]=phasefreqmodwpt(z,J0,kfilt,deltat,t);
% 
% calculate pure signal amplitudes and phases to put into variance formula
% 
phiD=phi(1,:); 
ampD=amp(1,:);
av56=varnu(ampD,phiD,sigmaeps,1,2,1);
% apply factor of 4 as we are using analytic transform TWICE
av56=4.*av56;
av3=sqrt(max(av56,zeros(1,length(signal))));
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% steps 2 and 3:
% now loop over realizations to 
% (a) simulate variance of inst freq 
% (b) repeatedly estimate the same using estimated params in the formula
%
for loop=1:200
%	noise(1:length(signal))=zeros;
	noise=sigmaeps.*randn(size(signal));
	x=signal+noise;
	input=x;
	y=hilbert(input); % make  analytic
	[phi,phi2,amp,vector]=phasefreqmodwpt(y,J0,kfilt,deltat,t);
	sigmahat0=(median(abs(vector(3,:))).*2.^(J0./2))./0.6745
	y=y.*dvector; % apply demodulation vector
	z=canalyticno2(y); % render new complex signal analytic
	[phi,phi2,amp,vector]=phasefreqmodwpt(z,J0,kfilt,deltat,t);
% 
% keep inst freq est -- will later find variance of these
	q=phi2(1,:); % est inst f from band n=1
%	eif(loop,:)=abs(q-qD).^2;
	eif(loop,:)=q;
% now estimate variance using estimated parameters
% noise var is estimated from band 3
	sigmahat=(median(abs(vector(3,:))).*2.^(J0./2))./0.6745
	vv=varnu(amp(1,:),phi(1,:),sigmahat./4,1,2,1);% sigmahat is aprox 4 sigmaeps
% apply factor of 4 as we are using analytic transform TWICE
	vv=4.*vv;
	veif(loop,:)=vv;
end
a1=std(eif,1,1); % simulated std dev of inst freq
av1=max(a1,zeros(1,length(x)));
av2=sqrt(max(mean(veif,1),zeros(1,length(x))));% std dev using estimated params


figure(1)
t1=(10:1014);
% sdev of `inst freq' estimates using actual amp and phase
subplot(3,1,1)
xlabel('')
plot(t1, av3(10:1014))
axis([0 1024 0 0.015])
% sdev from 200 sims using quantities estimated from data
subplot(3,1,2)
plot(t1,av2(10:1014))
axis([0 1024 0 0.015])
xlabel('')
% simulated sdev vs sdev from 200 sims using quantities estimated from data
subplot(3,1,3)
plot(t1, av1(10:1014))
axis([0 1024 0 0.015])
hold on
plot(t1,av2(10:1014),'--')

xlabel('time')

