%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
%
% Part of softare library HSpec ("Hilbert Spectrum")
% Free software written by  Sofia Olhede and Andrew Walden,
% Dept. of Mathematics, Imperial College, London SW7 2BZ, UK
% November 2004
%
% For terms of use and warranty disclaimer please refer to the
% README file
%
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%
function val=varnu(A,phi,sigma,comp,j,k)
%
% computes var(nu_{j,n,l}) for demodulation paper, 
% "A generalized demodulation approach to time-frequency projections for
% multicomponent signals" by S. Olhede and A.T. Walden
%  Proc R. Soc. Lond. A., under revision.
%
% where nu_{j,n,l} is instantaneous frequency at level j band n time l.
%
% A = inst amp
% phi=inst phase
% sigma=std dev 
% comp=1 for packets
% j = level
% k=n
%
% assumed differencing filter is [-1./12 8./12 0 -8./12 1/12]
%
n=length(phi);
t=1:n;
t1p=flop(t,1);% changes [1 2 3..] to [2 3 4 ..] wrapped
t2p=flop(t,2);
t1m=flop(t,-1);
t2m=flop(t,-2);
%
v=length(sigma);
if v==1
sigma=sigma.*ones(1,n);
end;
%
% the same lags (m-k) or (t-s) will be generated for different products of the
% filter coeffs r_m r_k; eg lag -1 for m=-2, k=-1 or m=1, k=2 with `weights'
% -1/12 * 8/12 and -8/12* 1/12 giving -16/144 in total: these are the values
% in the variable val at the bottom. Note this approach assumes local stationarity
% in amplitudes and phases simce we work out one case and weight it, whereas each
% case will show very small variation. Could sum each term, but 16 altogether
% (m=-2,-1,1,2) and same for k ......
covm4=covphi(A(t2m),A(t2p),phi(t2m),phi(t2p),t2m,t2p,comp,j,k,sigma);% lag -4
covm3=covphi(A(t2m),A(t1p),phi(t2m),phi(t1p),t2m,t1p,comp,j,k,sigma);
covm2=covphi(A(t1m),A(t1p),phi(t1m),phi(t1p),t1m,t1p,comp,j,k,sigma);
covm1=covphi(A(t2m),A(t1m),phi(t2m),phi(t1m),t2m,t1m,comp,j,k,sigma);
cov0=covphi(A(t),A(t),phi(t),phi(t),t,t,comp,j,k,sigma);
covp1=covphi(A(t1m),A(t2m),phi(t1m),phi(t2m),t1m,t2m,comp,j,k,sigma);
covp2=covphi(A(t1p),A(t1m),phi(t1p),phi(t1m),t1p,t1m,comp,j,k,sigma);
covp3=covphi(A(t2p),A(t1m),phi(t2p),phi(t1m),t2p,t1m,comp,j,k,sigma);
covp4=covphi(A(t2p),A(t2m),phi(t2p),phi(t2m),t2p,t2m,comp,j,k,sigma);% lag +4
% note (24^2*pi^2) = 144*(4 pi^2)
val=(-1.*covm4+16.*covm3-64.*covm2 -16.*covm1+ 130.*cov0 ...
-1.*covp4+16.*covp3-64.*covp2 -16.*covp1)./(24^2*pi^2);
